/**
 * Copyright (C) 2009 Brightcove, Inc.  All Rights Reserved.  No
 * use, copying or distribution of this work may be made except in
 * accordance with a valid license agreement from Brightcove, Inc.
 * This notice must be included on all copies, modifications and
 * derivatives of this work.
 *
 * Brightcove, Inc MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT
 * THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
 * NON-INFRINGEMENT. BRIGHTCOVE SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED
 * BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS
 * SOFTWARE OR ITS DERIVATIVES.
 *
 * "Brightcove" is a trademark of Brightcove, Inc.
 **/
package {

    import com.brightcove.core.utils.Tracer;
    import com.brightcove.fl.advertising.rules.AdRules;
    import com.brightcove.fl.advertising.rules.AdRulesContext;
    
    import flash.events.Event;
    import flash.events.IOErrorEvent;
    import flash.events.SecurityErrorEvent;
    import flash.net.URLLoader;
    import flash.net.URLRequest;
    import flash.system.Security;

    /**
    * This example ad rules SWF uses the built-in DFP ad translator 
    * and will only make a preroll ad call. This ad rules
    * logic is equivalent to the built-in BC preroll ad policy.
    * 
    * This example is just an example and should not be used in production
    * for real ad trafficking.
    */
    public class DFPExample extends AdRules {

        // NOTE for internal development at Brightcove.  Make sure to put changes to this file in the README

        private var context:AdRulesContext;
        private var adAPI:Object;
        private var experienceAPI:Object;
        private var videoAPI:Object;
        private var adServer:String;
        private var kvp:String;
        private var xmlLoader:URLLoader;
        
        private static const CONFIG_XML:String = "http://localhost:8080/cdn/adrules/DFPExample_config.xml";
        private static const XML_CONFIG:String = "xmlConfig";
        
        public function DFPExample() {
            // allow the Brightcove player SWFs to communicate with this SWF
            Security.allowDomain("admin.brightcove.com")         
        }
        
        /**
        * Load the player API modules.
        */
        override public function setAdRulesContext(pAdRulesContext:AdRulesContext):void {
            context = pAdRulesContext;
            context.moduleLoader.addEventListener(Event.COMPLETE, modulesLoaded);
            context.moduleLoader.loadModules();
        }
        
        /**
        * Once API modules have loaded, decide which xml config file to use (the default
        * or one specified by player-level key value pairs in the Advertising Module).
        */
        private function modulesLoaded(event:Event):void {
            context.moduleLoader.removeEventListener(Event.COMPLETE, modulesLoaded);
            
            // Now that the APIs are loaded in we can set module references.
            // We aren't actually using the APIs in this example, the below
            // is just for reference.
            adAPI = context.moduleLoader.getModule("advertising"); 
            experienceAPI = context.moduleLoader.getModule("experience");   
            videoAPI = context.moduleLoader.getModule("videoPlayer");
            
            // When an ad policy SWF is set in the Advertising Module,
            // the only ad policy field enabled is the player key-value pairs field. 
            var xmlConfigFile:String = findValue(adAPI.getAdPolicy().playerAdKeys, XML_CONFIG);
            if (!xmlConfigFile) xmlConfigFile = CONFIG_XML;
            
            // Load the xml config file that contains an ad server url and
            // any key-value pairs that should be passed with the ad call.
            xmlLoader = new URLLoader();
            xmlLoader.addEventListener(Event.COMPLETE, onXMLLoaded);
            xmlLoader.addEventListener(IOErrorEvent.IO_ERROR, onXMLIOError);
            xmlLoader.addEventListener(SecurityErrorEvent.SECURITY_ERROR, onXMLSecurityError);
            xmlLoader.load(new URLRequest(xmlConfigFile));
        }
        
        /**
        * Once the xml config file has been loaded, set the DFP translator.
        */
        private function onXMLLoaded(event:Event):void {
            xmlLoader.removeEventListener(Event.COMPLETE, onXMLLoaded);
            
            var xml:XML = new XML(event.target.data);
            adServer = xml.adServerURL[0] ? xml.adServerURL[0] : "";
            kvp = xml.kvp[0] ? xml.kvp[0] : "";
            
            // Set the DFP ad server URL and key/value pairs in the player.
            var adPolicy:Object = new Object();
            adPolicy.adServerURL = adServer;
            adPolicy.playerAdKeys = kvp;
            adAPI.setAdPolicy(adPolicy);
            
            // Function to set the built-in DFP translator as current translator.
            // 'adTranslatorLoaded' will then be called once the translator is ready.
            useDefaultDFPTranslator(true);
            
            // adRulesReady() MUST be called at some point in your ad SWF
            // or the player will hang. Calling this function tells the player
            // that the ad rules SWF is ready. 
            adRulesReady();
        }
        
        /**
        * Dispatched if the xml download is terminated.
        */
        private function onXMLIOError(event:IOErrorEvent):void {
            xmlLoader.removeEventListener(IOErrorEvent.IO_ERROR, onXMLIOError);
            Tracer.debug(this, event.toString(), Tracer.TYPE_AD);
            
            // adRulesReady() MUST be called at some point in your ad SWF
            // or the player will hang. Calling this function tells the player
            // that the ad rules SWF is ready. 
            adRulesReady();
        }
        
        /**
        * Dispatched if the xml can't be loaded because of security sandbox
        * issues.
        */
        private function onXMLSecurityError(event:SecurityErrorEvent):void {
            xmlLoader.removeEventListener(SecurityErrorEvent.SECURITY_ERROR, onXMLSecurityError);
            Tracer.debug(this, event.text, Tracer.TYPE_AD);
            
            // adRulesReady() MUST be called at some point in your ad SWF
            // or the player will hang. Calling this function tells the player
            // that the ad rules SWF is ready. 
            adRulesReady();
        }
        
        
        /**
         * Override the handler in AdRules.as that is called when an onload ad should
         * render. Request an ad now, simulating an onLoad insertion point.
         */
        override public function prerollCheck():void{
            
            callAdTranslator();
            // Let the player know that we are done checking for a preroll ad.
            // See AdRules.as doc for more information.
            prerollCheckComplete();
        }
        
        /**
         * Returns the value for the specified key. 
         */
        private function findValue(keyValues:String, key:String):String {
            if (! keyValues || keyValues.length == 0) return null;

            var i:int = keyValues.indexOf(key + "=");
            if (i != -1) {
                var j:int = keyValues.indexOf(";", i);
                if (j == -1) j = keyValues.length;
                return keyValues.substring(i + (key + "=").length, j);
            }
            
            return null;
        }

    }
}
