/**
 * Copyright (C) 2009 Brightcove, Inc.  All Rights Reserved.  No
 * use, copying or distribution of this work may be made except in
 * accordance with a valid license agreement from Brightcove, Inc.
 * This notice must be included on all copies, modifications and
 * derivatives of this work.
 *
 * Brightcove, Inc MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT
 * THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
 * NON-INFRINGEMENT. BRIGHTCOVE SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED
 * BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS
 * SOFTWARE OR ITS DERIVATIVES.
 *
 * "Brightcove" is a trademark of Brightcove, Inc.
 **/
package {

    import com.brightcove.api.APIModules;
    import com.brightcove.api.modules.AdvertisingModule;
    import com.brightcove.api.modules.CuePointsModule;
    import com.brightcove.api.modules.VideoPlayerModule;
    import com.brightcove.fl.advertising.api.BrightcoveModuleWrapper;
    import com.brightcove.fl.advertising.rules.AdRules;
    import com.brightcove.fl.advertising.rules.AdRulesContext;
    
    import flash.events.Event;
    import flash.events.IEventDispatcher;
    import flash.system.Security;

    /**
    * This example ad rules SWF loads an ad translator which then loads 
    * an ad SWF. This example shows how you can use a ad rules, an ad 
    * translator, and an ad SWF to serve and render ads in a Brightcove
    * player.
    * 
    * All three feature classes - ExampleAdRules.as (ad rules feature),
    * ExampleTranslatorWithAdSwf.as (ad translator feature), and ExampleAdSwfForAdRules.as
    * (ad SWF feature) - are compiled into one ad rules SWF, which replaces
    * the Brightcove built-in ad policy as set via the Advertising Module.
    * This particular example transfers all control from the Brightcove's 
    * built-in ad policy to the ad SWF feature, allowing for all ad requests and 
    * rendering to be handled by the ad SWF alone,
    * 
    * In order to compile this class, you must have bc_ads.swc in your 
    * external-library path. See the ad rules documentation for more details.
    * 
    * Changes to this file can be found in the README in AdRulesDevKit.zip
    */
    public class ExampleAdRules extends AdRules {

        // NOTE for internal development at Brightcove.  Make sure to put changes to this file in the README

        protected var context:AdRulesContext;
        protected var exampleAdSwf:ExampleAdSwfForAdRules;
        protected var adAPI:AdvertisingModule;
        protected var cuePointsAPI:CuePointsModule;
        protected var videoPlayerAPI:VideoPlayerModule;
        
        // wraps the adContext.moduleLoader so that bc_ads.swc and 
        // BrightcovePlayerAPI.swc can be used together in developing ad features.
        protected var moduleLoader:BrightcoveModuleWrapper;
        
        public function ExampleAdRules() {
            // Allow the Brightcove player SWFs to communicate with this SWF.
            Security.allowDomain("admin.brightcove.com")             
        }
        
        /**
        * Set the ad rules context object.
        */
        override public function setAdRulesContext(pAdRulesContext:AdRulesContext):void {
            context = pAdRulesContext;
            // Wrap adContext.moduleLoader in the BrightcoveModuleWrapper for use with both
            // development API swcs.
            moduleLoader = new BrightcoveModuleWrapper(context.moduleLoader as IEventDispatcher);
            
            // Set the ExampleTranslatorWithAdSwf translator as current translator
            // by compiling it in. Pass the ad SWF reference to the ad translator 
            // for display.
            exampleAdSwf = new ExampleAdSwfForAdRules();
            setAdTranslator(new ExampleTranslatorWithAdSwf(exampleAdSwf));
            
            // Instead of setting the translator directly, you could also call 
            // loadAdTranslator() here to load the translator from a URL. 
            // adTranslatorLoaded() will still be fired once the translator is 
            // loaded and ready for interaction.
            // loadAdTranslator("http://example.com/adtranslator.swf");
        }
        
        /**
        * Override the handler in AdRules.as that indicates whether or 
        * not the translator has finished loading. Load the player API modules
        * now.
        */
        override public function adTranslatorLoaded():void {
            moduleLoader.addEventListener(Event.COMPLETE, modulesLoaded);
            moduleLoader.loadModules();
        }
        
        /**
         * Called when the API modules have been loaded.
         */
        protected function modulesLoaded(pEvent:Event):void {
            moduleLoader.removeEventListener(Event.COMPLETE, modulesLoaded);

            // Now that the APIs are loaded in we can get the individual modules
            // for later use.
            adAPI = moduleLoader.getModule(APIModules.ADVERTISING) as AdvertisingModule;
            cuePointsAPI = moduleLoader.getModule(APIModules.CUE_POINTS) as CuePointsModule;
            videoPlayerAPI = moduleLoader.getModule(APIModules.VIDEO_PLAYER) as VideoPlayerModule;
            // Pass the advertising API module to the ad SWF so the API doesn't have to
            // be loaded every time an ad call is made. You could also pass other
            // API modules through to your ad SWF but, in this example, we only need 
            // the ad API in our ad SWF.
            exampleAdSwf.adAPI = adAPI;
            
            // adRulesReady() MUST be called at some point in your ad SWF
            // or the player will hang. Calling this function tells the player
            // that the ad rules SWF is ready. In this simple case, we wanted to make sure
            // all our API modules were loaded and ready to be used before we tell the
            // player that our SWF is ready.
            adRulesReady();
        }
        
        /**
        * Override the AdRules.as method. This is called whenever a preroll
        * ad can be played. For this example, we want to play prerolls on 
        * every video so execute 'callAdTranslator' here to play an ad at the 
        * preroll insertion, right before the content starts to play back.
        */
        override public function prerollCheck():void {
            
            // We also want to trigger midroll ads in this example so we
            // dynamically add cue points to the current video.
            // See 'midrollCheck()' below, as well.
            var cuePoints:Array = new Array({time:6, type:"ad", name:"6 sec cue", metadata:"This is an ad cue point at 6 seconds."},
                                            {time:12, type:"ad", name:"12 sec cue", metadata:"This is an ad cue point at 12 seconds."});
            var currentVideoID:Number = videoPlayerAPI.getCurrentVideo().id;
            cuePointsAPI.addCuePoints(currentVideoID, cuePoints);
            
            // callAdTranslator triggers "fetchAd" in ExampleTranslatorWithAdSwf.as
            callAdTranslator();
            // Let the player know that we are done checking for a preroll ad.
            // See AdRules.as doc for more information.
            prerollCheckComplete();
        }
        
        /**
        * Override the AdRules.as method. This is called whenever a midroll
        * ad can be played. Since we dynamically added cue points above in
        * 'prerollCheck' this should be triggered at 6 seconds and at 12 seconds
        * for every video that is played back. If ad cue points have been set on the 
        * video using the Advertising Module then even more midroll cue points may be fired
        * but we ignore those cue points in this example.
        */
        override public function midrollCheck():void {
            // You might want to check which insertion point you are in, just in case a publisher added
            // ad cue points through the BC Advertising Module that you don't want to recognize. This isn't
            // necessary for all implementations, of course, but is demonstrated below.
            var currentInsertion:Number = Math.round(videoPlayerAPI.getVideoPosition() as Number);
            if (currentInsertion == 6 || currentInsertion == 12) {
                // If the current insertion matches our dynamically added cue points
                // then trigger "fetchAd" in the ExampleTranslatorWithAdSwf.as.
                callAdTranslator();
            }
           
            // Tell the player you are done checking for the midroll ad.
            midrollCheckComplete()
        }
        
        /**
        * Override the AdRules.as method. This is called whenever a postroll
        * ad can be played. For this example, we want to play postrolls
        * on every video so execute 'callAdTranslator' here to play an
        * ad at the postroll insertion, right after the content finishes playing.
        */
        override public function postrollCheck():void {
            // Go ahead and clear out all the dynamically added midroll cue points in 
            // the current video before moving on.
            var currentVideoID:Number = videoPlayerAPI.getCurrentVideo().id;
            cuePointsAPI.clearAdCuePoints(currentVideoID);
            
            // callAdTranslator triggers "fetchAd" in ExampleTranslatorWithAdSwf.as
            callAdTranslator();
            // Let the player know that we are done checking for a postroll ad.
            // See AdRules.as doc for more information.
            postrollCheckComplete();
        }

    }
}

