/**
 * Copyright (C) 2009 Brightcove, Inc.  All Rights Reserved.  No
 * use, copying or distribution of this work may be made except in
 * accordance with a valid license agreement from Brightcove, Inc.
 * This notice must be included on all copies, modifications and
 * derivatives of this work.
 *
 * Brightcove, Inc MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT
 * THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
 * NON-INFRINGEMENT. BRIGHTCOVE SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED
 * BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS
 * SOFTWARE OR ITS DERIVATIVES.
 *
 * "Brightcove" is a trademark of Brightcove, Inc.
 **/
package {

    import com.brightcove.api.modules.AdvertisingModule;
    import com.brightcove.fl.advertising.adSwf.AdSwf;
    import com.brightcove.fl.advertising.adSwf.AdSwfContext;
    
    import flash.display.Sprite;
    import flash.events.Event;
    import flash.events.TimerEvent;
    import flash.system.Security;
    import flash.utils.Timer;

    /**
     * An example ad SWF for an ad translator.  This ad SWF does not call a real ad server or 
     * load a real ad, but it shows you how to create a real ad SWF for an ad translator, 
     * specifically, via an ad rules SWF.
     */
    public class ExampleAdSwfForAdRules extends AdSwf {
        
        // NOTE for internal development at Brightcove.  Make sure to put changes to this file in the README
        
        // timer for deciding when the ad should end
        private var timer:Timer;
        
        // visual ad
        private var ad:Sprite;
        
        // reference to the Advertising API
        // for this example, the ad API is set in the ad rules SWF (Example.as)
        public var adAPI:AdvertisingModule;

        public function ExampleAdSwfForAdRules() {
            // Set 'pauseVideoPlayer' to true so that this behaves like a linear
            // ad format.
            config.pauseVideoPlayer = true;
            // Set 'completeOnVideoComplete' to false so that postrolls render like
            // a linear ad format.
            config.completeOnVideoComplete = false;
            // allow the Brightcove player SWFs to communicate with this SWF
            Security.allowDomain("admin.brightcove.com");
        }
        
        /**
         * Called when a new ad should be displayed.
         */
        override public function displayAd(adSwfContext:AdSwfContext):void {
            showAd();
        }
        
        /**
        * Called when the ad is finished.
        */
        override public function adComplete():void {
            if (timer) {
                timer.removeEventListener(TimerEvent.TIMER_COMPLETE, timerComplete);
                timer = null;
            }
            
            if (contains(ad)) {
                removeChild(ad);
            }
            ad = null;
            adAPI.showSponsorMessage(false);
            
            super.adComplete();
        }
        
        /**
        * Renders ad.
        */
        private function showAd():void {
            // Get our fake ad instead of getting it from a Loader
            ad = new FakeAd(0x333333, 0x5050555, "I'm an Ad SWF loaded using an Ad Translator via an Ad Rules SWF!");

            // Add the ad to the display
            addChild(ad);
            
            // After we have the correct width/height for the ad, make sure this Sprite has the same dimensions
            width = ad.width;
            height = ad.height;
             
            // End the preroll ad after 6 seconds.
            timer = new Timer(6000, 1);
            timer.addEventListener(TimerEvent.TIMER_COMPLETE, timerComplete);
            timer.start();
            
            adAPI.showSponsorMessage(true);
        }
        
        /**
         * Called to end the ad.
         */
        private function timerComplete(event:Event):void {
            adComplete();
        }  
    }
}
