/**
 * Copyright (C) 2008 Brightcove, Inc.  All Rights Reserved.  No
 * use, copying or distribution of this work may be made except in
 * accordance with a valid license agreement from Brightcove, Inc.
 * This notice must be included on all copies, modifications and
 * derivatives of this work.
 *
 * Brightcove, Inc MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT
 * THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
 * NON-INFRINGEMENT. BRIGHTCOVE SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED
 * BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS
 * SOFTWARE OR ITS DERIVATIVES.
 *
 * "Brightcove" is a trademark of Brightcove, Inc.
 **/
package {

    import com.brightcove.api.APIModules;
    import com.brightcove.api.modules.AdvertisingModule;
    import com.brightcove.api.modules.ExperienceModule;
    import com.brightcove.api.modules.VideoPlayerModule;
    import com.brightcove.fl.advertising.adSwf.AdSwf;
    import com.brightcove.fl.advertising.adSwf.AdSwfContext;
    import com.brightcove.fl.advertising.adSwf.AdSwfMediaEvent;
    import com.brightcove.fl.advertising.api.BrightcoveModuleWrapper;
    
    import flash.display.Sprite;
    import flash.events.Event;
    import flash.events.IEventDispatcher;
    import flash.events.TimerEvent;
    import flash.system.Security;
    import flash.utils.Timer;

    /**
     * An example ad SWF.  This ad SWF does not load a real ad, but shows you show to create
     * a real ad SWF by drawing a simple fake ad.  It uses all of the ad SWF APIs, and not 
     * all of this may be needed within your own ad SWF.  It can show either a non-linear ad 
     * (the default) or a linear ad (when pauseVideo is set to true). 
     * 
     * In order to compile this class, you must have bc_ads.swc in your 
     * external-library path.  See the ad SWF documentation for more details.
     * 
     * Changes to this file can be found in the README in AdSwfDevKit.zip 
     */
    public class ExampleAdSwf extends AdSwf {

        // NOTE for internal development at Brightcove.  Make sure to put changes to this file in the README
                
        // whether to pause the video while the ad is playing
        protected var pauseVideo:Boolean = false;
        
        // whether to show only the sponsor message while the ad is playing
        protected var showSponsorMessage:Boolean = false;

        // whether to disable all Lists when an ad is playing and pauseVideo is true.  This can 
        // be used to make sure that no other video can be clicked on when an ad is playing
        protected var disableLists:Boolean = false;
        
        // timer for deciding when the ad should end
        protected var timer:Timer;
        
        // information about the player and functions for the ad SWF
        protected var adContext:AdSwfContext;
        
        // wraps the adContext.moduleLoader so that bc_ads.swc and 
        // BrightcovePlayerAPI.swc can be used together in developing ad features.
        private var moduleLoader:BrightcoveModuleWrapper;
        
        // videoPlayer API module.  See the API documentation for more information on its use
        protected var videoPlayer:VideoPlayerModule;

        // advertising API module.  See the API documentation for more information on its use
        protected var advertising:AdvertisingModule;

        // experience API module.  See the API documentation for more information on its use
        protected var experience:ExperienceModule;

        // the ad duration  
        protected var adDuration:Number;
        
        // whether to pause the video manually while the ad is playing.  Use pauseVideo
        // instead of this unless you have both a non-linear and linear portion to an ad.
        protected var pauseVideoManually:Boolean;
        
        // element that can be put in ad XML to set pauseVideo
        protected static const PAUSE_VIDEO:String = "pauseVideo";

        // element that can be put in ad XML to set adDuration
        protected static const AD_SWF_DURATION:String = "adSWFDuration";

        // element that can be put in ad XML to set disableLists
        protected static const DISABLE_LISTS:String = "disableLists";
        
        // element that can be put in ad XML to set controls mode to show only
        // the sponsor message for a linear ad
        protected static const SPONSOR_MESSAGE:String = "sponsorMessage";
        
        // element that can be put in ad XML to pause the video manually through the API
        protected static const PAUSE_VIDEO_MANUALLY:String = "pauseVideoManually";                

        public function ExampleAdSwf() {
            // allow the Brightcove player SWFs to communicate with this SWF
            Security.allowDomain("admin.brightcove.com");             
        }

        /**
         * Called before displayAd() when a new ad should be displayed.  This can be used to set up
         * AdSwfConfig for the current ad.  If the settings will be the same for every ad, then
         * the AdSwfConfig could instead be set up in the constructor.
         */
        override public function configureAd(pAdContext:AdSwfContext):void {

            // Set the pauseVideoPlayer value in AdSwfConfig as determined by the pauseVideo
            // settings.  If set to true, this will pause the main video and enable some 
            // events to listen to
            if (pAdContext.adSwfFormat.additionalElements.hasOwnProperty(PAUSE_VIDEO)) {
                pauseVideo = new Boolean(pAdContext.adSwfFormat.additionalElements[PAUSE_VIDEO]);    
            }
            config.pauseVideoPlayer = pauseVideo;
                        
            // The rest of the AdSwfConfig values aren't really needed in here, as they are set to the
            // defaults, but we include this to show where you could set them. 
            config.completeOnVideoComplete = true;
            config.completeOnVideoChange = true;
            config.resizeOnMenuOpen = true;
            config.resizeOnFullScreen = true;
            config.keepVisibleAfterComplete = false;              
            config.customStartStopEvents = false;
            
            // make sure to call configAdComplete() when you are done with your configuration.  You must 
            // call this function if you override configureAd() otherwise the ad will never play.
            configureAdComplete();
        }
        
        /**
         * Called when a new ad should be displayed
         */
        override public function displayAd(adContext:AdSwfContext):void {

            this.adContext = adContext;
            
            // Wrap adContext.moduleLoader in the BrightcoveModuleWrapper for use with both
            // development API swcs.
            this.moduleLoader = new BrightcoveModuleWrapper(adContext.moduleLoader as IEventDispatcher);

            trace("Beginning the display of the example SWF ad");                                  
                                    
            // before we do anything else, we load the API modules for potential use
            moduleLoader.addEventListener(Event.COMPLETE, modulesLoaded);
            moduleLoader.loadModules();
        }
        
        /**
         * Called when the API modules have been loaded
         */
        private function modulesLoaded(event:Event):void {
            
            moduleLoader.removeEventListener(Event.COMPLETE, modulesLoaded);
            
            // Now we can get a reference to a module we'd like to use.  We can grab 
            // more modules if they are needed, as the ad SWF can use any of the modules 
            // that can normally be used for player API development.
            videoPlayer = moduleLoader.getModule(APIModules.VIDEO_PLAYER) as VideoPlayerModule;
            advertising = moduleLoader.getModule(APIModules.ADVERTISING) as AdvertisingModule; 
            experience = moduleLoader.getModule(APIModules.EXPERIENCE) as ExperienceModule;
                                                
            // Normally you wouldn't call a function like adLoaded directly.  Instead, you would
            // be using the Loader class, which loads the real ad and then call adLoaded() functionality
            // after the result has been returned
            adLoaded();
        }

        /**
         * Setup that usually occurs after a Loader has returned a result  
         */
        protected function adLoaded():void {

            // Get our fake ad instead of getting it from a Loader
            var ad:Sprite = new FakeAd(0x000000, 0xf0ff00," I'm an Ad SWF!");
            
            // You can place the ad on the screen in the location you'd like it to be.  This may also be
            // done by the loaded ad.  If it's done by the loaded ad, then the ad SWF may not be resized
            // properly when the menu is shown or when in full-screen mode
            x = videoPlayer.getX() + 15;
            y = videoPlayer.getY() + 15;

            // add the ad to the display
            addChild(ad);
            
            // After we have the correct width/height for the ad, make sure this Sprite has the same dimensions
            width = ad.width;
            height = ad.height;            

            // After the ad is loaded and displayed, make a trackStart impression call
            adContext.markTrackStartImpression();
              
            // End the ad after 30 seconds (or the time specified as an argument).  This code is  
            // usually not needed and an ad ending will be managed a different way.  Usually the ad 
            // will send an event or disappear when it's done.  Note that you should call adComplete() 
            // when the ad is done, which is done in timerComplete in this example
            adDuration = 0;
            var additionalElements:Object = adContext.adSwfFormat.additionalElements;
            if (additionalElements.hasOwnProperty(AD_SWF_DURATION)) {
                adDuration = new Number(additionalElements[AD_SWF_DURATION]);  
            }
            if (adDuration < 100) {
                adDuration = 15000;
            }              

            timer = new Timer(adDuration / 100, 100);
            timer.addEventListener(TimerEvent.TIMER_COMPLETE, timerComplete);
            timer.start();
            
            if (additionalElements.hasOwnProperty(PAUSE_VIDEO_MANUALLY)) {
                // normally in an ad SWF it would not make sense to pause the video manually
                // right here.  Using the pauseVideo route will provide a more consistent 
                // experience, especially for prerolls.  But if you have a non-linear ad that
                // can transition into a linear ad, this is the way to do things when the linear
                // part begins.
                pauseVideoManually = new Boolean(additionalElements[PAUSE_VIDEO_MANUALLY]);
                if (pauseVideoManually) {
                    videoPlayer.pause();
                }
                timer.addEventListener(TimerEvent.TIMER, onTimeEvent);
            }
            
            if (pauseVideo || pauseVideoManually) {                
                if (additionalElements.hasOwnProperty(SPONSOR_MESSAGE)) {
                    // Show the sponsor message in controls during the ad play. 
                    // Indicates a linear ad that user cannot control using player controls.
                    // Make sure to also set this to false when the ad is complete 
                    showSponsorMessage = new Boolean(additionalElements[SPONSOR_MESSAGE]); 
                    if (showSponsorMessage) {
                        advertising.showSponsorMessage(true); 
                    }
                } else {
                    // Enable the control of the video controls and then set up the event listeners that
                    // will be used to listen for the different video events
                    advertising.allowThirdPartyControl(true);
                    addEventListener(AdSwfMediaEvent.RESUME, onResume);
                    addEventListener(AdSwfMediaEvent.PAUSE, onPause);
                    addEventListener(AdSwfMediaEvent.VOLUME_CHANGE, onVolumeChanged);
                }
                // get the value for disableLists and disable the lists if necessary
                if (additionalElements.hasOwnProperty(DISABLE_LISTS)) {
                    disableLists = new Boolean(additionalElements[DISABLE_LISTS]);
                }
                if (disableLists) {
                    enableLists(false);
                }                
            }                
        }
        
        /**
         * Send the time to thirdPartyTime when it won't be set through the normal mediaPosition route
         */
        protected function onTimeEvent(event:TimerEvent):void {
            advertising.setThirdPartyTime(mediaDuration - mediaPosition);
        }
        
        /**
         * Disables or enables the List and related elements (TabBar, TileList) 
         * within the player 
         */
        protected function enableLists(enable:Boolean):void {
            if (! experience) return;
            var tabBars:Array = experience.getElementsByType("TabBar");
            enableInArray(enable, tabBars);
            var lists:Array = experience.getElementsByType("List");
            enableInArray(enable, lists);
            lists = experience.getElementsByType("TileList");
            enableInArray(enable, lists);
        }
        
        /**
         * For all elements in the given Array, call setEnabled() with the given
         * enabled value
         */
        private function enableInArray(enable:Boolean, elementArr:Array):void {
            if (elementArr) {
                for (var i:int = 0; i < elementArr.length; i++) {
                    var element:Object = elementArr[i];
                    element.setEnabled(enable);
                }
            }
        }
        
        /**
         * When pauseVideo is true, called when "play" button in video controls is clicked 
         * or played via the API
         */
        protected function onResume(event:AdSwfMediaEvent):void {
            trace("Resuming ad");
            timer.start();
            // Dispatch an ad resume event in case anybody is listening for it
            adResume();
        }
        
        /**
         * When pauseVideo is true, called when "pause" button in video controls is clicked 
         * or paused via the API.
         */
        protected function onPause(event:AdSwfMediaEvent):void {
            trace("Pausing ad");
            timer.stop();
            // Dispatch an ad resume event in case anybody is listening for it
            adPause();
        }        
        
        /**
        * When pauseVideo is true, called when user changes the volume using the volume controls 
        * or uses the video API 
        */
        protected function onVolumeChanged(event:AdSwfMediaEvent):void {
            // We don't do anything here, as we don't really have a real ad that needs a
            // change in volume
            trace("Volume change for ad: " + event.volume);
        }
        
        /**
         * Override mediaDuration to provide the correct ad duration for video events
         */
        override public function get mediaDuration():Number {
            return adDuration / 1000;
        } 

        /**
         * Override mediaDuration to provide the correct ad duration for video events and
         * to change the time remaining
         */        
        override public function get mediaPosition():Number {
            if (! timer) {
                return super.mediaPosition;
            }
            else {
                return (timer.currentCount * (adDuration / 100)) / 1000;
            } 
        }
        
        /**
         * Called to end the ad.  adComplete() is called because the ad is done
         */
        protected function timerComplete(event:Event):void {
            adComplete();
        }        
        
        /**
         * You can override the adComplete() function to cleanup anything when the ad is done.  
         * The player may call adComplete() for you, if you have completeOnVideoChange or 
         * completeOnVideoCompete set to true 
         */
        override public function adComplete():void {
            // stop the Timer since we're already at ad complete
            if (timer) {
                timer.removeEventListener(TimerEvent.TIMER_COMPLETE, timerComplete);
                timer.removeEventListener(TimerEvent.TIMER, onTimeEvent);
                timer = null;
            }
           
            if (pauseVideo || pauseVideoManually) {
                if (showSponsorMessage) {
                    // set showSponsorMessage back to its original state, if called.
                    advertising.showSponsorMessage(false);
                } else {
                    // set allowThirdPartyControl back to its original state, which must be done
                    // at adComplete if set to true earlier
                    advertising.allowThirdPartyControl(false);
                    // remove all event listeners
                    removeEventListener(AdSwfMediaEvent.RESUME, onResume);
                    removeEventListener(AdSwfMediaEvent.PAUSE, onPause);
                    removeEventListener(AdSwfMediaEvent.VOLUME_CHANGE, onVolumeChanged); 
                }               
                // reenable the Lists for clicking
                if (disableLists) {
                    enableLists(true);
                }
            }            
            if (pauseVideoManually) {
                videoPlayer.play();
            }                        
            
            // if you do override adComplete(), make sure you call super.adComplete() within it.
            // Note that once adComplete is called, the ad SWF loses communication with the ad SWF
            // infastructure.  So sending a custom event (such as one for adComplete) won't work.
            // If you are sending a custom adComplete event, it's also usually better to simply set 
            // config.customStartStopEvents to false before calling super.adComplete() and then back
            // to true afterwards.  This allows for the timing of the adComplete event to be exactly 
            // the same as it usually is.
            super.adComplete();
        }
        
        /**
         * Overriding this function to provide a response for the player API call
         * getCurrentAdProperties().
         * 
         * These values are hardcoded for demonstration purposes, but for a real implementation 
         * it would be expected that many of the values would be retrieved dynamically.
         * 
         * @return An object containing current ad properties.
         */
        override public function get currentAdProperties():Object {
            var properties:Object = new Object();
            properties["mVideoURL"] = "ExampleAdSwf.swf";
            properties["videoURL"] = "ExampleAdSwf.swf";
            properties["duration"] = 10;
            properties["adType"] = "videoad";
            properties["formatName"] = "videoAd";
            properties["additionalKeys"] = new Object();
            properties["videoClickURL"] = null;
            properties["trackPointTime"] = 5;
            /*
            properties["cuePointInsertion"] = "Pre-roll";
            properties["cuePointName"] = "Pre-roll";
            properties["cuePointMetadata"] = null;
            properties["trackPauseURLs"] = 
            "http://www.brightcove.com/tracker?trackPoint=pause";
            properties["trackResumeURLs"] = 
            "http://www.brightcove.com/tracker?trackPoint=resume";
            properties["trackMuteURLs"] = 
            "http://www.brightcove.com/tracker?trackPoint=mute";
            properties["trackUnmuteURLs"] = 
            "http://www.brightcove.com/tracker?trackPoint=unmute";
            properties["trackFullscreenURLs"] = 
            "http://www.brightcove.com/tracker?trackPoint=fullscreen";
            properties["trackFirstQuartileURLs"] = 
            "http://www.brightcove.com/tracker?trackPoint=firstQuartile";
            properties["trackThirdQuartileURLs"] = 
            "http://www.brightcove.com/tracker?trackPoint=third";
            properties["trackExitFullscreenURLs"] = 
            "http://www.brightcove.com/tracker?trackPoint=exitFullscreen";
            properties["clickTrackingURLs"] = 
            "http://www.brightcove.com/tracker?trackPoint=clickTracking";
            properties["trackMidURLs"] = 
            "http://www.brightcove.com/tracker?trackPoint=mid";
            properties["trackPointURLs"] = 
            "http://www.brightcove.com/tracker?trackPoint=point";
            properties["trackStartURLs"] = 
            "http://www.brightcove.com/tracker?trackPoint=start";  
            properties["trackEndURLs"] = 
            "http://www.brightcove.com/tracker?trackPoint=end";
            */
            return properties;
        }
    }
}