/**
 * Copyright (C) 2008 Brightcove, Inc.  All Rights Reserved.  No
 * use, copying or distribution of this work may be made except in
 * accordance with a valid license agreement from Brightcove, Inc.
 * This notice must be included on all copies, modifications and
 * derivatives of this work.
 *
 * Brightcove, Inc MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT
 * THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
 * NON-INFRINGEMENT. BRIGHTCOVE SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED
 * BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS
 * SOFTWARE OR ITS DERIVATIVES.
 *
 * "Brightcove" is a trademark of Brightcove, Inc.
 **/
package {

    import com.brightcove.fl.advertising.AdContext;
    import com.brightcove.fl.advertising.translation.AdTranslation;
    
    import flash.system.Security;

    /**
     * An ad translator to use with ExampleAdSwf or any other ad swf for testing.  You must pass in the 
     * location of the ad swf as a key/value pair using the key of 'adSwf'.   
     * 
     * This translator does not actually call an ad server but instead returns an ad with the 
     * ad swf referenced.
     * 
     * In order to compile this class, you must have bc_ads.swc in your external-library path.
     * See the ad translation documentation for more details.
     * 
     * Changes to this file can be found in the README in AdSwfDevKit.zip 
     */
    public class ExampleTranslator extends AdTranslation {
    
       // NOTE for internal development at Brightcove.  Make sure to put changes to this file in the README

        private static const DEFAULT_AD_SWF:String = "http://admin.brightcove.com/adswf/Example.swf";
         
        public function ExampleTranslator() {
            // allow the Brightcove player SWFs to communicate with this SWF
            Security.allowDomain("admin.brightcove.com");         
        } 
        
        /**
         * Fetch the ad, which in this case means we use a precanned ad  
         **/
        override public function fetchAd(context:AdContext):void {

            var adSwf:String = null;
            // find the 'adswf' key in all key/values            
            var arr:Array = context.playerKeyValues ? context.playerKeyValues.split(";") : [];
            arr.concat(context.titleKeyValues ? context.titleKeyValues.split(";") : []);
            arr.concat(context.insertionKeyValues ? context.insertionKeyValues.split(";") : []);
            
            for (var i:int = 0; i < arr.length; i++) {
                var keyValue:Array = String(arr[i]).split("=");
                if (keyValue.length > 1 && String(keyValue[0]).toLowerCase() == "adswf") {
                    adSwf = keyValue[1];
                }
            }
            if (adSwf == null) {
                trace("No adSwf value found in the player key/values.  This key should be set to use this translator.");
                trace("Using the default adSwf location: " + DEFAULT_AD_SWF);
                adSwf = DEFAULT_AD_SWF; 
            }
            else {
                trace("About to load adSwf " + adSwf); 
            }
            
            var fakedResponse:XML = new XML("<adSwf version='1'>" + 
                "<adSwfURL>" + adSwf + "</adSwfURL>" + 
                "<optionalArg>ad data</optionalArg>" + 
                "</adSwf>");
            setAdXML(fakedResponse); 
        }
    }
}
