/**
 * Copyright (C) 2008 Brightcove, Inc.  All Rights Reserved.  No
 * use, copying or distribution of this work may be made except in
 * accordance with a valid license agreement from Brightcove, Inc.
 * This notice must be included on all copies, modifications and
 * derivatives of this work.
 *
 * Brightcove, Inc MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT
 * THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
 * NON-INFRINGEMENT. BRIGHTCOVE SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED
 * BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS
 * SOFTWARE OR ITS DERIVATIVES.
 *
 * "Brightcove" is a trademark of Brightcove, Inc.
 **/
package {
    import com.brightcove.fl.advertising.AdContext;
    import com.brightcove.fl.advertising.adSwf.AdSwf;
    import com.brightcove.fl.advertising.adSwf.AdSwfContext;
    import flash.display.Sprite;
    import flash.events.Event;
    import flash.events.TimerEvent;
    import flash.net.URLRequest;
    import flash.net.sendToURL;
    import flash.system.Security;
    import flash.utils.Timer;

    /**
     * An example ad SWF for an ad translator.  This ad SWF does not call a real ad server or 
     * load a real ad, but it shows you show to create a real ad SWF for an ad translator.
     * 
     * This ad SWF is focused on the functionality related to loading an ad SWF in a translator.  
     * It does not show the full functionality that ad SWFs provide.  To see a fuller ad SWF example,
     * see AdSwfDevKit.zip
     * 
     * In order to compile this class, you must have bc_ads.swc in your 
     * external-library path.  See the ad SWF documentation for more details.
     * 
     * Changes to this file can be found in the README in AdTranslatorDevKit.zip 
     */
    public class ExampleAdSwfForTranslators extends AdSwf {

        // NOTE for internal development at Brightcove.  Make sure to put changes to this file in the README

        // constants for events that we dispatch
        public static const AD_LOADED:String = "adLoaded";
        public static const AD_NOT_LOADED:String = "adNotLoaded";
        
        // timer for deciding when the ad should end
        private var timer:Timer;

        // information about the player and functions for the ad translation request
        private var adTransContext:AdContext;
        
        // information about the player and functions for the ad SWF
        private var adSwfContext:AdSwfContext;

        public function ExampleAdSwfForTranslators():void {
            // allow the Brightcove player SWFs to communicate with this SWF
            Security.allowDomain("admin.brightcove.com");             

            // Often you'd load in another third-party SWF in the constructor, an ad library SWF for 
            // the ad server you're working with
            
            // To note again, this ad SWF is focused on the functionality related to loading an 
            // ad SWF in a translator.  It does not show the full functionality that 
            // ad SWFs provide.  To see a fuller ad SWF example, see AdSwfDevKit.zip            
        }                           

        /**
         * Called to request the ad.  This function is specific to the ad SWF format but is simply the
         * function we created for the ad translator to call 
         */                
        public function requestAd(adContext:AdContext):void {

            this.adTransContext = adContext;
            
            // Normally you'd interact with the ad server here through the ad library SWF or custom
            // server calls.  Since this is an example where we're just going to display a fake ad,
            // we dispatch an event saying that we have an ad
            dispatchEvent(new Event(AD_LOADED));
        }
        
        /**
         * Called when a new ad should be displayed
         */
        override public function displayAd(adContext:AdSwfContext):void {
            
            this.adSwfContext = adSwfContext;

            trace("Beginning the display of the example SWF ad");

            // Usually you'd use adContext.moduleLoader.loadModules() here, so that you 
            // have access to the APIs.  Because this is an example that's just focused on 
            // loading an ad SWF in an ad translator, we'll skip this.  See Example.as
            // in AdSwfDevKit.zip for a more full-featured example

            // Get our fake ad instead of getting it from a Loader
            var ad:Sprite = new FakeAd(0x333333, 0x5050555, "I'm an Ad SWF loaded using an Ad Translator!");

            // Add the ad to the display
            addChild(ad);
            
            // After we have the correct width/height for the ad, make sure this Sprite has the same dimensions
            width = ad.width;
            height = ad.height;
            
            // Send a start impression.  This would normally be handled by the ad SWF library you're using.
            // We do this in the example for internal Brightcove testing  
            sendStartImpression();        
              
            // End the ad after 30 seconds.  This usally isn't needed and an ad ending will be managed a
            // different way.  Usually the ad will send an event or disappear when it's done.  Note that
            // you should call adComplete() when the ad is done, which is done in timerComplete in this
            // example  
            timer = new Timer(30000, 1);
            timer.addEventListener(TimerEvent.TIMER_COMPLETE, timerComplete);
            timer.start();
        }        

        /**
         * Send a start impression to the URL specified in adSwfImpression.  This is really just used
         * for internal Brightcove testing, and you'll probably have this handled by the ad SWF library
         * you're using.  The URL does not include any real details about the request, like a video id
         */
        private function sendStartImpression():void {
            // find the 'adSwfImpression' key in all key/values
            var adSwfImpression:String = null;            
            var allKVP:String=adTransContext.playerKeyValues + ";" +
                              adTransContext.titleKeyValues +  ";" +
                              adTransContext.insertionKeyValues +  ";" +
                              adTransContext.additionalAdTargetingParams;
            var arr:Array = allKVP ? allKVP.split(";") : [];
            
            for (var i:int = 0; i < arr.length; i++) {
                var keyValue:Array = String(arr[i]).split("=");
                if (keyValue.length > 1 && String(keyValue[0]).toLowerCase() == "adswfimpression") {
                    adSwfImpression = keyValue[1];
                }
            }
            
            if (adSwfImpression != null && adSwfImpression != "") {
                try {
                    sendToURL(new URLRequest(adSwfImpression + ";trackPoint=start" +
                                             ";plID=" + adTransContext.playerID +
                                             ";ttID=" + adTransContext.titleID + 
                                             ";" + allKVP));
                                             
                    trace("calling adSwfImpression");
                } catch (e:Error) {
                    trace(e.getStackTrace());
                }
            } else {
                trace ("could not find key adswfimpression");
            }
        }
        
        /**
         * Called to end the ad.  adComplete() is called because the ad is done
         */
        private function timerComplete(event:Event):void {
            // stop the Timer since we're already at ad complete
            if (timer) {
                timer.removeEventListener(TimerEvent.TIMER_COMPLETE, timerComplete);
                timer = null;
            }
            adComplete();
        }        
    }
}
