/**
 * Copyright (C) 2008 Brightcove, Inc.  All Rights Reserved.  No
 * use, copying or distribution of this work may be made except in
 * accordance with a valid license agreement from Brightcove, Inc.
 * This notice must be included on all copies, modifications and
 * derivatives of this work.
 *
 * Brightcove, Inc MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT
 * THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
 * NON-INFRINGEMENT. BRIGHTCOVE SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED
 * BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS
 * SOFTWARE OR ITS DERIVATIVES.
 *
 * "Brightcove" is a trademark of Brightcove, Inc.
 **/
package {

    import com.brightcove.fl.advertising.AdContext;
    import com.brightcove.fl.advertising.translation.AdTranslation;

    import flash.events.Event;
    import flash.system.Security;

    /**
     * An example ad translator which uses an ad SWF.  This translator shows how you can interact with 
     * an ad SWF in order to serve ads.  
     * 
     * In order to compile this class, you must have bc_ads.swc in your external-library path.
     * See the ad tag translation documentation for more details.
     * 
     * Changes to this file can be found in the README in AdTranslatorDevKit.zip 
     */
    public class ExampleWithAdSwf extends AdTranslation {
    
        // NOTE for internal development at Brightcove.  Make sure to put changes to this file in the README
        
        // the ad SWF we've loaded to really do the ad translation and to display the ad
        private var exampleAdSwf:ExampleAdSwfForTranslators;
        
        // the current AdContext
        private var context:AdContext;
        
        public function ExampleWithAdSwf() {

            // allow the Brightcove player SWFs to communicate with this SWF
            Security.allowDomain("admin.brightcove.com");             
            
            // The translator is loaded when the player is loaded.  You could add event hooks into the player
            // here or load anything else that would be needed for ad translation.
           trace("Ad translator with ad SWF has been loaded.");              
            
            // Create the ad Swf and set a local reference it
            exampleAdSwf = new ExampleAdSwfForTranslators();

            // Let AdTranslation know about the ad SWF
            adSwf = exampleAdSwf;            
            
            // Instead of setting adSwf directly, you could also call loadAdSwf() here to load the ad SWF from
            // a URL.  You'd then also have to override adSwfLoaded() to get a reference to the ad SWF once
            // it was loaded 
            //loadAdSwf(AD_SWF);
        }
        
        /**
         * Construct ad call and fetch the ad.  
         **/
        override public function fetchAd(context:AdContext):void {
            
            trace("Within fetchAd() in example ad translator with ad SWF");            
            this.context = context;

            if (exampleAdSwf == null) {
                // we can't display an ad if we don't have an ad SWF, so we stop here.
                setNoAd();
            }
            else {
                exampleAdSwf.addEventListener(ExampleAdSwfForTranslators.AD_LOADED, adLoaded);
                exampleAdSwf.addEventListener(ExampleAdSwfForTranslators.AD_NOT_LOADED, noAdLoaded);
                
                // Rather than handling the translation logic in here, we have the ad SWF handle the requesting
                // of the ad 
                exampleAdSwf.requestAd(context);
            }
        }
                
        /**
         * Event listener for when ExampleAdSwfForTranslators has loaded an ad 
         */
        private function adLoaded(event:Event):void {
            exampleAdSwf.removeEventListener(ExampleAdSwfForTranslators.AD_LOADED, adLoaded);
            exampleAdSwf.removeEventListener(ExampleAdSwfForTranslators.AD_NOT_LOADED, noAdLoaded);

            trace("Ad was loaded in example ad translator with ad SWF");
            
            // Tell the player that an ad has been loaded and that the ad translator's ad SWF will display
            // the ad
            useAdSwf();
        }

        /**
         * Event listener for when ExampleAdSwfForTranslators doesn't have an ad
         */
        private function noAdLoaded(event:Event):void {
            exampleAdSwf.removeEventListener(ExampleAdSwfForTranslators.AD_LOADED, adLoaded);
            exampleAdSwf.removeEventListener(ExampleAdSwfForTranslators.AD_NOT_LOADED, noAdLoaded);

            trace("No ad was loaded in example ad translator with ad SWF");
            
            // Tell the player that we're done
            setNoAd(); 
        }
    }
}
